@@ -92,7 +92,7 @@ create_window_widget (Window *window)
   gtk_image_set_pixel_size (GTK_IMAGE (window_image), 32);
 
   check_image = gtk_image_new_from_icon_name ("object-select-symbolic");
-  gtk_widget_hide (check_image);
+  gtk_widget_set_visible (check_image, FALSE);
 
   row = adw_action_row_new ();
   adw_action_row_add_prefix (ADW_ACTION_ROW (row), window_image);
@@ -117,7 +117,7 @@ create_monitor_widget (LogicalMonitor *logical_monitor)
   GList *l;
 
   check_image = gtk_image_new_from_icon_name ("object-select-symbolic");
-  gtk_widget_hide (check_image);
+  gtk_widget_set_visible (check_image, FALSE);
 
   row = adw_action_row_new ();
 
@@ -152,7 +152,7 @@ create_virtual_widget (void)
   GtkWidget *row;
 
   check_image = gtk_image_new_from_icon_name ("object-select-symbolic");
-  gtk_widget_hide (check_image);
+  gtk_widget_set_visible (check_image, FALSE);
 
   row = adw_action_row_new ();
   adw_action_row_add_suffix (ADW_ACTION_ROW (row), check_image);
@@ -189,8 +189,7 @@ update_windows_list (ScreenCastWidget *widget)
   GtkListBox *window_list = GTK_LIST_BOX (widget->window_list);
   GtkWidget *toplevel;
   GtkWidget *child;
-  GList *windows;
-  GList *l;
+  GPtrArray *windows;
 
   while ((child = gtk_widget_get_first_child (GTK_WIDGET (window_list))) != NULL)
     gtk_list_box_remove (window_list, child);
@@ -200,9 +199,9 @@ update_windows_list (ScreenCastWidget *widget)
     return;
 
   windows = shell_introspect_get_windows (widget->shell_introspect);
-  for (l = windows; l; l = l->next)
+  for (size_t i = 0; windows && i < windows->len; i++)
     {
-      Window *window = l->data;
+      Window *window = g_ptr_array_index (windows, i);
       GtkWidget *window_widget;
 
       if (should_skip_window (window, GTK_WINDOW (toplevel)))
@@ -292,9 +291,9 @@ connect_windows_changed_listener (ScreenCastWidget *widget)
   g_assert (!widget->windows_changed_handler_id);
   widget->windows_changed_handler_id =
     g_signal_connect (widget->shell_introspect,
-                          "windows-changed",
-                          G_CALLBACK (on_windows_changed),
-                          widget);
+                      "windows-changed",
+                      G_CALLBACK (on_windows_changed),
+                      widget);
   shell_introspect_ref_listeners (widget->shell_introspect);
 }
 
@@ -302,9 +301,8 @@ static void
 disconnect_windows_changed_listener (ScreenCastWidget *widget)
 {
   g_assert (widget->windows_changed_handler_id);
-  g_signal_handler_disconnect (widget->shell_introspect,
-                               widget->windows_changed_handler_id);
-  widget->windows_changed_handler_id = 0;
+  g_clear_signal_handler (&widget->windows_changed_handler_id,
+                          widget->shell_introspect);
   shell_introspect_unref_listeners (widget->shell_introspect);
 }
 
@@ -486,18 +484,18 @@ screen_cast_widget_init (ScreenCastWidget *widget)
   g_signal_connect (widget->source_type, "notify::visible-child",
                     G_CALLBACK (on_stack_switch),
                     widget);
-  g_signal_connect (widget->monitor_list, "row-activated",
-                    G_CALLBACK (on_row_activated),
-                    NULL);
-  g_signal_connect (widget->window_list, "row-activated",
-                    G_CALLBACK (on_row_activated),
-                    NULL);
-  g_signal_connect (widget->monitor_list, "selected-rows-changed",
-                    G_CALLBACK (on_selected_rows_changed),
-                    widget);
-  g_signal_connect (widget->window_list, "selected-rows-changed",
-                    G_CALLBACK (on_selected_rows_changed),
-                    widget);
+  g_signal_connect_object (widget->monitor_list, "row-activated",
+                           G_CALLBACK (on_row_activated), NULL,
+                           G_CONNECT_DEFAULT);
+  g_signal_connect_object (widget->window_list, "row-activated",
+                           G_CALLBACK (on_row_activated), NULL,
+                           G_CONNECT_DEFAULT);
+  g_signal_connect_object (widget->monitor_list, "selected-rows-changed",
+                           G_CALLBACK (on_selected_rows_changed), widget,
+                           G_CONNECT_DEFAULT);
+  g_signal_connect_object (widget->window_list, "selected-rows-changed",
+                           G_CALLBACK (on_selected_rows_changed), widget,
+                           G_CONNECT_DEFAULT);
 
   widget->display_state_tracker = display_state_tracker_get ();
   widget->monitors_changed_handler_id =
@@ -509,9 +507,6 @@ screen_cast_widget_init (ScreenCastWidget *widget)
 
   update_monitors_list (widget);
   update_windows_list (widget);
-
-  gtk_widget_show (widget->monitor_list);
-  gtk_widget_show (widget->window_list);
 }
 
 GtkWidget *
@@ -538,7 +533,7 @@ screen_cast_widget_set_app_id (ScreenCastWidget *widget,
       if (info)
         display_name = g_app_info_get_display_name (info);
       else
-        display_name = g_strdup (app_id);
+        display_name = app_id;
       monitor_heading = g_strdup_printf (_("Select monitor to share with %s"),
                                          display_name);
       window_heading = g_strdup_printf (_("Select window to share with %s"),
@@ -575,16 +570,16 @@ screen_cast_widget_set_source_types (ScreenCastWidget     *screen_cast_widget,
   screen_cast_widget->source_types = source_types;
 
   if (source_types & SCREEN_CAST_SOURCE_TYPE_MONITOR)
-    gtk_widget_show (screen_cast_widget->monitor_selection);
+    gtk_widget_set_visible (screen_cast_widget->monitor_selection, TRUE);
 
   if (source_types & SCREEN_CAST_SOURCE_TYPE_WINDOW)
-    gtk_widget_show (screen_cast_widget->window_selection);
+    gtk_widget_set_visible (screen_cast_widget->window_selection, TRUE);
 
   if (source_types & SCREEN_CAST_SOURCE_TYPE_VIRTUAL)
-    gtk_widget_show (screen_cast_widget->monitor_selection);
+    gtk_widget_set_visible (screen_cast_widget->monitor_selection, TRUE);
 
   if (__builtin_popcount (source_types) > 1)
-    gtk_widget_show (screen_cast_widget->source_type_switcher);
+    gtk_widget_set_visible (screen_cast_widget->source_type_switcher, TRUE);
 
   update_monitors_list (screen_cast_widget);
 }
@@ -656,7 +651,7 @@ screen_cast_widget_set_persist_mode (ScreenCastWidget      *screen_cast_widget,
   screen_cast_widget->persist_mode = persist_mode;
 
   gtk_widget_set_visible (GTK_WIDGET (screen_cast_widget->persist_check),
-                                      persist_mode != SCREEN_CAST_PERSIST_MODE_NONE);
+                          persist_mode == SCREEN_CAST_PERSIST_MODE_PERSISTENT);
 }
 
 ScreenCastPersistMode
